local sprites=require('sprites/sprites')
local images
local batch1,batch2,batch3
local mode=1
local id
local cnt=1000
local scrWidth,scrHeight=love.graphics.getDimensions()
local spriteSize=160

local function getModeName(m)
  if m==1 then
    return 'multiple draws'
  elseif m==2 then
    return 'stream batch'
  elseif m==3 then
    return 'dynamic batch'
  elseif m==4 then
    return 'static batch'
  end
end

local function getPos()
  return math.random(0,scrWidth-spriteSize),math.random(0,scrHeight-spriteSize)
end

local function getRandomQuad()
  return sprites[math.random(#sprites)].quad
  end

local function initBatch()
  batch1=love.graphics.newSpriteBatch(sprites[1].texture,cnt,'stream')
  batch2=love.graphics.newSpriteBatch(sprites[1].texture,cnt,'dynamic')
  batch3=love.graphics.newSpriteBatch(sprites[1].texture,cnt,'static')
  
  --dynamic batch
  id={}
  batch2:clear()
  for i=1,cnt do
    id[#id+1] = batch2:add(getRandomQuad())    
  end
  --static batch
  batch3:clear()
  for i=1,cnt do
    batch3:add(getRandomQuad(),getPos())
  end
end

function love.load()
  images={}
  for _,v in ipairs(love.filesystem.getDirectoryItems('images')) do    
    images[#images+1] = love.graphics.newImage('images/'..v)
  end
  
  initBatch()
end

function love.draw()
  if mode==1 then --multiple draws
    for i=1,cnt do
      love.graphics.draw(images[math.random(#images)],getPos())      
    end    
  elseif mode==2 then --stream batch
    love.graphics.draw(batch1)
  elseif mode==3 then --dynamic batch
    love.graphics.draw(batch2)
  elseif mode==4 then --static batch
    love.graphics.draw(batch3)
  end
  love.graphics.print('Space-change draw mode. Up/down-change texture quantity.'..
    '\nfps: '..love.timer.getFPS()..
    '\nmode: '..getModeName(mode)..
    '\ntextures: '..(cnt))
end

function love.keypressed(key)
  if key=='up' then
    cnt=cnt+1000
    initBatch()
  elseif key=='down' then
    cnt=cnt-1000
    initBatch()
  elseif key=='space' or key==' ' then
    mode = mode+1
    if mode>4 then
      mode=1
    end
  end  
end

function love.mousepressed(x,y)
  if x<scrWidth*1/3 then
    cnt=cnt-1000
    initBatch()
  elseif x>=scrWidth*1/3 and x<=scrWidth*2/3 then
    mode = mode+1
    if mode>4 then
      mode=1
    end
  elseif x>scrWidth*2/3 then
    cnt=cnt+1000
    initBatch()
  end

end

function love.resize(w,h)
  scrWidth,scrHeight=love.graphics.getDimensions()
  initBatch()
end

function love.update(dt)
  --change stream batch
  batch1:clear()
  for i=1,cnt do
    batch1:add(getRandomQuad(),getPos())
  end

  --change dynamic batch
  for i=1,cnt do
    batch2:set(id[i],getRandomQuad(),getPos())    
  end
end

